@@ -23,6 +23,8 @@ from DeepMTP.utils.model_utils import EarlyStopping
 from DeepMTP.utils.eval_utils import get_performance_results
 
 class TwoBranchDotProductModel(nn.Sequential):
+	'''Implements a two branch neural network that uses a dot product to combine the two embeddings.
+	'''	
 	def __init__(self, config, instance_branch_model, target_branch_model):
 		super(TwoBranchDotProductModel, self).__init__()
 		self.instance_branch_model = instance_branch_model
@@ -36,6 +38,8 @@ class TwoBranchDotProductModel(nn.Sequential):
 		return output
 
 class TwoBranchMLPModel(nn.Sequential):
+	'''Implements a two branch neural network that uses an MLP on top of the two branches. The two embedding vectors are just concatenated
+	'''
 	def __init__(self, config, instance_branch_model, target_branch_model):
 		super(TwoBranchMLPModel, self).__init__()
 		self.instance_branch_model = instance_branch_model
@@ -53,9 +57,9 @@ class TwoBranchMLPModel(nn.Sequential):
 
 
 class DeepMTP:
-
-	def __init__(self, config, checkpoint_dir=None):
-
+	''' Implements the training and inference logic of the DeepMTP framework. 
+	'''
+	def __init__(self, config, instance_branch_model=None, target_branch_model=None, checkpoint_dir=None):
 		self.checkpoint_dict = None
 		self.wandb_run = None
 		self.tensorboard_logger = None
@@ -102,15 +106,23 @@ class DeepMTP:
 			target_branch_output_dim = self.config['target_branch_nodes_per_layer'][-1] if isinstance(self.config['target_branch_nodes_per_layer'], list) else self.config['target_branch_nodes_per_layer']
 		
 		# define models that will be used in the two branches
-		if self.config['instance_branch_architecture'] == 'MLP':
-			self.instance_branch_model = MLP(self.config, self.config['instance_branch_input_dim'], instance_branch_output_dim, self.config['instance_branch_nodes_per_layer'], self.config['instance_branch_layers'], self.config['dropout_rate'], self.config['batch_norm'])
-		elif self.config['instance_branch_architecture'] == 'CONV':
-			self.instance_branch_model = ConvNet(self.config, self.config['instance_branch_input_dim'], instance_branch_output_dim, self.config['instance_branch_conv_architecture'], self.config['instance_branch_conv_architecture_version'], self.config['instance_branch_conv_architecture_last_layer_trained'], self.config['instance_branch_conv_architecture_dense_layers'])
-		
-		if self.config['target_branch_architecture'] == 'MLP':
-			self.target_branch_model = MLP(self.config, self.config['target_branch_input_dim'], target_branch_output_dim, self.config['target_branch_nodes_per_layer'], self.config['target_branch_layers'], self.config['dropout_rate'], self.config['batch_norm'])
-		elif self.config['target_branch_architecture'] == 'CONV':
-			self.target_branch_model = ConvNet(self.config, self.config['target_branch_input_dim'], target_branch_output_dim, self.config['target_branch_conv_architecture'], self.config['target_branch_conv_architecture_version'], self.config['target_branch_conv_architecture_last_layer_trained'], self.config['target_branch_conv_architecture_dense_layers'])
+		if instance_branch_model is None:
+			if self.config['instance_branch_architecture'] == 'MLP':
+				self.instance_branch_model = MLP(self.config, self.config['instance_branch_input_dim'], instance_branch_output_dim, self.config['instance_branch_nodes_per_layer'], self.config['instance_branch_layers'], self.config['dropout_rate'], self.config['batch_norm'])
+			elif self.config['instance_branch_architecture'] == 'CONV':
+				self.instance_branch_model = ConvNet(self.config, self.config['instance_branch_input_dim'], instance_branch_output_dim, self.config['instance_branch_conv_architecture'], self.config['instance_branch_conv_architecture_version'], self.config['instance_branch_conv_architecture_last_layer_trained'], self.config['instance_branch_conv_architecture_dense_layers'])
+		else:
+			# support for custom instance branch models
+			self.instance_branch_model = instance_branch_model(self.config)
+
+		if target_branch_model is None:
+			if self.config['target_branch_architecture'] == 'MLP':
+				self.target_branch_model = MLP(self.config, self.config['target_branch_input_dim'], target_branch_output_dim, self.config['target_branch_nodes_per_layer'], self.config['target_branch_layers'], self.config['dropout_rate'], self.config['batch_norm'])
+			elif self.config['target_branch_architecture'] == 'CONV':
+				self.target_branch_model = ConvNet(self.config, self.config['target_branch_input_dim'], target_branch_output_dim, self.config['target_branch_conv_architecture'], self.config['target_branch_conv_architecture_version'], self.config['target_branch_conv_architecture_last_layer_trained'], self.config['target_branch_conv_architecture_dense_layers'])
+		else:
+			# support for custom target branch models
+			self.target_branch_model = target_branch_model(self.config)
 
 		# initalize the two-branch neural network
 		if self.config['enable_dot_product_version']:
@@ -364,6 +376,16 @@ class DeepMTP:
 
 
 		self.deepMTP_model = self.early_stopping.best_model        
+
+		# log the performance validation results of the best model
+		results_to_log = {'best_val_'+m+'_'+av: self.early_stopping.best_performance_results['val_'+m+'_'+av] for m in self.config['metrics'] for av in self.config['metrics_average'] if 'val_'+m+'_'+av in self.early_stopping.best_performance_results}
+		if self.wandb_run is not None:
+			self.wandb_run.log(results_to_log)
+		if self.tensorboard_logger is not None:
+			for k, v in results_to_log.items():
+				self.tensorboard_logger.add_scalar(
+					k.replace('_', '/'), v, epoch
+				)
 		# training is done (either completed all epochs or early stopping kicked in). Now testing starts using the best model
 		if self.config['verbose']: print('Starting testing... ', end='')
 		test_results = self.inference(self.deepMTP_model, test_dataloader, 'test', epoch, verbose=self.config['verbose'])
